package paas_golang

import (
	"bytes"
	"encoding/json"
	"fmt"
	"github.com/google/uuid"
	"paas-golang/busi"
	"paas-golang/crypto"
	"paas-golang/http"
	"paas-golang/macros"
	"paas-golang/sign"
	"strconv"
	"time"
)

type ZZYClient struct {
	accessKeyID     string
	accessSecretKey string
	region          macros.PAAS_REGION
	ver             string

	// Subscribe
	// 订阅业务
	Subscribe *busi.Subscribe

	// Media
	// 媒体业务
	Media *busi.Media

	// Trans
	// 信令转发业务
	Trans *busi.Trans

	// Bind
	// 绑定业务
	Bind *busi.Bind

	// Device
	// 设备相关业务(控制命令、信息查询等)
	Device *busi.Device

	// Event
	// 事件业务
	Event *busi.Event

	// Video
	// 云存业务
	Video *busi.Video

	// Video
	// 云图片业务
	Picture *busi.Picture

	// Preset
	// 预置位业务
	Preset *busi.Preset

	// AI
	// AI业务
	AI *busi.AI
}

type ZZYRespone struct {
	RequestID string `json:"RequestID"`
	Message   string `json:"Message"`
	Code      uint32 `json:"Code,string"`
}

func (r *ZZYRespone) Error() string {
	b, _ := json.Marshal(r)
	return string(b)
}

func CreateZZYClient(accessKeyID, accessSecretKey string, region macros.PAAS_REGION) *ZZYClient {
	if accessKeyID == "" || accessSecretKey == "" {
		panic("Access Key ID or Access Secret Key is nil")
	}

	cli := &ZZYClient{
		accessKeyID:     accessKeyID,
		accessSecretKey: accessSecretKey,
		region:          region,
	}

	cli.Subscribe = busi.InitSubscribe(cli)
	cli.Media = busi.InitMedia(cli)
	cli.Trans = busi.InitTrans(cli)
	cli.Bind = busi.InitBind(cli)
	cli.Device = busi.InitDevice(cli)
	cli.Event = busi.InitEvent(cli)
	cli.Video = busi.InitVideo(cli)
	cli.Picture = busi.InitPicture(cli)
	cli.Preset = busi.InitPreset(cli)
	cli.AI = busi.InitAI(cli)

	return cli
}

func (cli *ZZYClient) RemoteRequestZZYServer(path macros.PAAS_PATH, m map[string]interface{}, action macros.PAAS_OPENAPI_ACTION, v interface{}) error {

	addr := macros.EndpointMap[cli.region]

	b, _ := json.Marshal(m)

	timestamp := strconv.Itoa(int(time.Now().Unix()))
	nonce := uuid.New().String()

	body, _ := crypto.AesEncryptOutBase64(b, []byte(cli.accessKeyID), []byte(cli.accessSecretKey))

	fmt.Println("body: ", body)
	signature := sign.PaaSOpenInterfaceSignatureV1(cli.accessKeyID, cli.accessSecretKey, nonce, body, timestamp, string(action), cli.ver)

	header := make(map[string]string)
	header["AccessKey"] = cli.accessKeyID
	header["Timestamp"] = timestamp
	header["Signature"] = signature
	header["Nonce"] = nonce
	header["Ver"] = cli.ver
	header["Action"] = string(action)

	b, err := http.PostHttpApplicationJsonRequest(addr, string(path), bytes.NewBufferString(body), http.WithHeaderMap(header))

	decodeB, _ := crypto.Base64AesDecodeToByte(string(b), []byte(cli.accessKeyID), []byte(cli.accessSecretKey))

	fmt.Printf("b: %s, err: %v\n", decodeB, err)

	return &ZZYRespone{}
}
