package http

import (
	"crypto/tls"
	"errors"
	"fmt"
	"io"
	"io/ioutil"
	"net"
	"net/http"
	"strings"
	"time"
)

type REQUEST_METHOD string

const (
	REQUEST_METHOD_POST REQUEST_METHOD = "POST"
	REQUEST_METHOD_GET  REQUEST_METHOD = "GET"
	//REQUEST_TYPE_PUT     REQUEST_TYPE = "PUT"
	//REQUEST_TYPE_DELETE  REQUEST_TYPE = "DELETE"
)

func PostHttpApplicationJsonRequest(addr string, path string, body io.Reader, opts ...Option) ([]byte, error) {
	addr = fmt.Sprintf("http://%s", addr)
	return BaseApplicationJsonRequest(addr, path, REQUEST_METHOD_POST, body, opts...)
}
func PostHttpsApplicationJsonRequest(addr string, path string, body io.Reader, opts ...Option) ([]byte, error) {
	addr = fmt.Sprintf("https://%s", addr)
	return BaseApplicationJsonRequest(addr, path, REQUEST_METHOD_POST, body, opts...)
}

func BaseApplicationJsonRequest(addr string, path string, method REQUEST_METHOD, body io.Reader, opts ...Option) ([]byte, error) {
	url := fmt.Sprintf("%s/%s", strings.TrimSuffix(addr, "/"), strings.TrimPrefix(path, "/"))
	return baseApplicationJsonRequest(url, method, body, opts...)
}

func baseApplicationJsonRequest(url string, method REQUEST_METHOD, body io.Reader, opts ...Option) ([]byte, error) {

	var options = Options{
		ResponseHeaderTimeout: time.Second * 3,
		HeaderMap:             nil,
		TimeOut:               time.Second * 3,
		KeepAlive:             time.Second * 3,
		TLSHandshakeTimeout:   time.Second * 3,
		DisableKeepAlives:     true,
	}

	for _, opt := range opts {
		opt(&options)
	}

	respBody, err := sendApplicationJsonRequest(url, method, body, options)
	if err != nil {
		return nil, err
	}
	return respBody, nil
}

func sendApplicationJsonRequest(url string, method REQUEST_METHOD, body io.Reader, options Options) ([]byte, error) {
	tr := &http.Transport{Dial: (&net.Dialer{
		Timeout:   options.TimeOut,
		KeepAlive: options.KeepAlive}).Dial,
		ResponseHeaderTimeout: options.ResponseHeaderTimeout,
		ExpectContinueTimeout: 1 * time.Second,
		TLSClientConfig:       &tls.Config{InsecureSkipVerify: options.InsecureSkipVerify},
		TLSHandshakeTimeout:   options.TLSHandshakeTimeout,
		IdleConnTimeout:       90 * time.Second,
		MaxIdleConns:          100,
		DisableKeepAlives:     options.DisableKeepAlives}
	conn := &http.Client{Transport: tr}

	req, err := http.NewRequest(string(method), url, body)
	if err != nil {
		return nil, err
	}

	req.Header.Set("Content-Type", "application/json;charset=utf-8")
	if options.HeaderMap != nil {
		for k, v := range options.HeaderMap {
			req.Header.Set(k, v)
		}
	}

	resp, err := conn.Do(req)

	if resp != nil && resp.Body != nil {
		defer resp.Body.Close()
	}

	if err != nil {
		return nil, err
	}

	if resp.StatusCode != http.StatusOK {
		return nil, errors.New(resp.Status)
	}

	rspBody, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}
	return rspBody, nil
}
